# Music Sampler


## Description

Music Sampler permet de pré-programmer des transitions musicales, qui peuvent ensuite être lancées à l'aide d'un simple appui sur une touche.


## Pré-requis et installation

Il faut avoir ffmpeg d'installé. Pour cela, il faut installer le paquet `libav-tools` :

```
sudo apt-get install libav-tools
```

Si vous utilisez la version compilée de Music Sampler, il n'y a pas d'installation nécessaire.

## Utilisation

Tout le travail consiste à préparer les transitions dans le fichier de configuration config.yml

Lancer ensuite le programme dans le dossier où se situe le fichier. Une fenêtre représentant un clavier apparaît. Les touches programmées apparaissent d'abord légèrement grisées, lorsque les musiques associées sont en cours de chargement, ou pas encore chargées. Le rond orange dans le coin du clavier devient vert lorsque tout est chargé.

Appuyer sur une touche déclenche les actions associées à cette touche (affichées dans le cadre en bas à gauche). Cliquer sur la touche affiche les actions associées à la touche mais ne les déclenche pas. Les deux autres cadres montrent respectivement un historique des touches appuyées et la liste des musiques en train d'être jouées.

## Configurer les touches

Le fichier config.yml utilise la syntaxe yaml. Les catégories et sous-catégories sont gérées par l'indentation par des espaces (mais PAS par des tabulations !).
le `#` est un symbole de commentaire : tout ce qui suit ce symbole sur une ligne est ignoré.

Le fichier contient plusieurs sections :
```
aliases:
...

music_properties:
...


key_properties:
...


keys:
...
```


### `music_properties` : propriétés des musiques

Cette section sert à définir des propriétés globales des musiques.

#### Exemples

```
  "music1.mp3":
    name: My favorite music
    gain: 1.4
```
La musique "music1.mp3" est désignée par le nom "My favorite music". Elle est chargée à 140% de son volume normal.

```
  "music2.mp3":
    gain: 0.7
```
La musique "music2.mp3" est chargée à 70% de son volume normal.

#### Liste des options possibles
- `name: My music` La musique sera désignée  (dans les actions, dans le terminal) comme "My music" au lieu du chemin du fichier. Par exemple le cadre des actions affichera "starting « My music » at volume 100%". Attention, cela ne fait pas office d'alias dans le fichier de configuration (voir la section *aliases*). 
- `gain: x` Charge la musique avec un gain de x (multiplicatif). Utiliser la commande "volume" pour changer ponctuellement le volume (0 à 100%) au cours de l'écoute.

### `key_properties` : affichage des touches

Cette section sert à décrire l'affichage à l'écran des touches : couleur et texte. Par défaut, une touche "attribuée" à une ou plusieurs actions s'affiche en vert.

#### Exemples

```
  'ESC':
    description:
      - 
      - STOP !
    color: [255, 0, 0]
```
La touche échap est de couleur rouge, et le texte "STOP !" est affiché sur la deuxième ligne

#### Liste des options possibles
- `description` : le texte qui s'affiche, à côté du "nom" de la touche. Il faut mettre un tirer pour une ligne de texte (pas de retour à la ligne automatique). La première ligne correspond à celle de la lettre associée à la touche, aussi il n'est pas absurde de la laisser vide (voir l'exemple ci-dessus). Sur un écran de taille raisonnable, on peut compter 3 lignes (incluant la première) pour une touche "standard".
- `color: [r,g,b]` : la couleur de la touche. r, g et b sont les proportions de rouge, vert et bleu, et doivent être des entiers entre 0 et 255.


### `keys` : actions sur les touches

Cette section sert à décrire, pour chaque touche, la liste des actions successives. Notez qu'à l'exception de `wait` (voir plus bas), les actions sont exécutées les unes à la suite des autres, sans attendre que la précédente soit terminée.


#### Exemples

```
'a':
  - play: 
      file: "music1.mp3"
      volume: 70
  - wait:
      duration: 5
  - stop:
      file: "music1.mp3"
      fade_out 2
```
Lance la musique "music1.mp3" à 70% de son volume max, puis au bout de 5 secondes coupe la musique avec un fondu de 2 secondes.

```
'b':
  - stop: 
      file "music1.mp3"
      fade_out 5
      wait: false
  - play:
      file "music2.mp3"
      fade_in 5
```
Effectue un fondu enchaîné de 5 secondes entre "music1.mp3" et "music2.mp3"

```
'c':
  - stop: 
      file "music1.mp3"
      fade_out 5
      wait: true
  - wait:
      duration: 2
  - play:
      file "music2.mp3"
  - seek:
      file "music2.mp3"
      delta: false
      value: 60
```
Coupe la musique "music1.mp3" avec un fondu de 5 secondes, attend la fin du fondu, puis attend deux secondes et lance la musique "music2.mp3", au temps d'une minute.

```
'd':
  - volume: 
      file "music1.mp3"
      value: 50
  - play:
      file "noise.mp3"
      loop: 1
  - wait:
      duration 2
  - volume:
      file "music1.mp3"
      value: 100
```
Baisse temporairement (pour 2 secondes) le volume de "music1.mp3" pendant que le son "noise.mp3" est joué par dessus (joué deux fois).

```
'e':
  - pause:
      file: "music1.mp3"
  - wait: 
      duration: 10
  - unpause:
      file: "music1.mp3"
  - seek:
      file: "music1.mp3"
      delta: true
      value: 5
```
Met en pause la musique "music1.mp3" pour 10 secondes et la relance après, en avançant de 5 secondes dans la musique.

#### Liste des actions possibles: 
- `play` : joue une musique. Paramètres :
    * `file: "music.mp3"` précise la musique jouée (chemin relatif).
    * `fade_in x` (facultatif) lance la musique avec un fade in de x secondes.
    * `volume x` (facultatif) la musique doit être jouée à x% de son volume max (x doit être entre 0 et 100, 100 par défaut)
    * `loop: x` (facultatif) la musique doit être répétée x fois. Indiquer -1 pour la répéter indéfiniment. Attention, x est le nombre de répétitions, donc pour lire trois fois la musique, mettre `loop: 2`
- `stop` : arrête une musique donnée. Paramètres :
    * `file: "music.mp3"` (facultatif) précise la musique à stopper. Si aucune musique n'est précisée, le `stop` s'applique à toutes les musiques.
    * `fade_out x` (facultatif) stoppe la musique avec un fondu de x secondes.
    * `wait: true/false` (par défaut : false) dans le cas d'un fondu, attendre la durée du fondu pour faire les actions suivantes. Si la musique s'arrêtait naturellement avant la fin du fondu, l'attente se termine lorsque la musique se termine naturellement. FIXME
- `volume` : change le volume d'une musique donnée. Paramètres :
    * `file: "music.mp3"` précise la musique. Si aucune musique n'est précisée, la modification s'applique au volume global.
    * `delta: true/false` le volume doit il être précisé en absolu (false, option par défaut), ou en relatif (true), voir plus bas.
    * `value: x` Si delta est à false, met le volume à x% du volume max (x doit être entre 0 et 100).
Ce facteur est appliqué à la musique déjà chargée en mémoire (voir section "propriétés"), donc le 100% fait référence au volume de chargement.
Si delta est à true, applique un modificateur de x% au volume (x doit être un entier signé).
- `pause` : met en pause une musique. Paramètres :
    * `file: "music.mp3"` précise la musique à mettre en pause.
- `unpause` : relance une musique mise en pause (là où elle en était). Paramètres :
    * `file: "music.mp3"` précise la musique à relancer.
- `wait` : attend un temps donné. Paramètres :
    * `duration: x` attend x secondes
Notez une fois enore que `wait` est la seule action qui attend d'avoir terminé pour lancer la commane suivante, toutes les autres sont lancées successivement mais sans attendre (donc quasiment simultanément).
- `seek` : permet d'aller à un endroit précis dans une musique. Paramètres :
    * `file: "music.mp3"` précise la musique. Si aucune musique n'est précisée, l'action s'applique à toutes les musiques.
    * `delta: true/false` Si delta est true, le temps est relatif. Si delta est false (par défaut ?), le temps est absolu, voir plus bas.
    * `value: x` Si delta est true, alors fait avancer de x secondes dans la musique (reculer si x est négatif). Si delta est false, alors la lecture se place à x secondes du début. Si la musique est en train de faire un fondu au départ, le fondu est oublié et la musique est jouée au volume normal. Si la musique est en train de se terminer en fondu, le "seek" est ignoré.

### `aliases` : définir des alias

Il est possible de définir des alias pour les différents objets du fichier yml.

#### Exemples

``
